import click
import http.server
import os
import pathlib
import socketserver


class MagicHTMLHandler(http.server.SimpleHTTPRequestHandler):
    def do_GET(self):
        self.rewrite_endpoint()
        return super().do_GET()

    def rewrite_endpoint(self):
        path = self.translate_path(self.path)
        new_path = self.path
        if os.path.exists(path + ".html"):
            new_path += ".html"
        elif os.path.exists(path + "/index.html"):
            new_path += "/index.html"
        else:
            return
        print("REWROTE {} => {}".format(self.path, new_path))
        # noinspection PyAttributeOutsideInit
        self.path = new_path


@click.command("serve")
@click.option("--root", required=True, help="Directory to serve")
@click.option("--port", default=8020, type=int, help="Port to serve on")
def main(root: str, port: str):
    root = pathlib.Path(root).expanduser().resolve()
    os.chdir(str(root))
    handler = MagicHTMLHandler
    socketserver.TCPServer.allow_reuse_address = True
    httpd = socketserver.TCPServer(("", port), handler)
    print("Serving CONSOLE on {}".format(port))
    httpd.serve_forever()



import bs4
from collections import ChainMap
import jinja2
import shutil
from pathlib import Path
from typing import MutableMapping, Mapping, Union

DEFAULT_MAPP_OUTPUT_PATH = "/static/mapp.min.js"

binary_suffixes = {
    "." + suffix
    for suffix in "gif img ico jpeg jpg png".split()
    if suffix.strip()  # guard against extra spaces
}


def default_is_binary(filename):
    """Used to tell the renderer which files to skip"""
    return Path(filename).suffix in binary_suffixes


def safe_context(context: Mapping=None) -> ChainMap:
    return ChainMap({}, (context or {}))


def extract_head(soup: bs4.BeautifulSoup, context: MutableMapping) -> None:
    head = None
    head_sections = soup.find_all("head")
    if head_sections:
        if len(head_sections) > 1:
            raise ValueError("html must have at most 1 <head> section.")
        head = head_sections[0].renderContents().decode("utf-8")
    context.setdefault("head", head)


def extract_main(soup: bs4.BeautifulSoup, context: MutableMapping) -> None:
    main_sections = soup.find_all("main")
    if len(main_sections) != 1:
        raise ValueError("html must have exactly 1 <main> section.")
    main = main_sections[0]
    context.setdefault("main", main.renderContents().decode("utf-8"))


def prettify_html(rendered: str) -> str:
    # TODO find a proper html prettify that doesn't add significant whitespace
    # return bs4.BeautifulSoup(rendered, "html.parser").prettify()
    return rendered


def extract_scripts(soup: bs4.BeautifulSoup, context: MutableMapping) -> None:
    # Find top-level only, don't grab scripts in main twice
    scripts = soup.find_all("script", recursive=False)
    scripts = "\n".join((element.prettify()) for element in scripts)
    context.setdefault("scripts", scripts)


class Renderer:
    def __init__(self, environment: jinja2.Environment, src_dir: Union[str, Path], out_dir: Union[str, Path]) -> None:
        self.environment = environment
        self.src_dir = Path(src_dir).expanduser()
        self.out_dir = Path(out_dir).expanduser()
        self.out_dir.mkdir(parents=True, exist_ok=True)

    def render_scaffolding(self, context: Mapping=None) -> None:
        """Required files to ensure mapp functions properly"""
        context = safe_context(context)
        self.render_template(name="_dynamicRoutes.json", context=context)
        self.render_template(name="_prefetchManifest.json", context=context)
        self.render_template(name="static/vendor/mapp.min.js", out_path="static/js/mapp.min.js", context=context)

    def render_template(self, name: str, out_path: Union[str, Path]=None, context: Mapping=None) -> None:
        if out_path is None:
            out_path = name
        context = safe_context(context)

        template = self.environment.get_template(name)
        rendered = template.render(context)

        # fix templating ugliness
        if name.endswith(".html"):
            rendered = prettify_html(rendered)

        dst = self.out_dir / out_path
        dst.parent.mkdir(parents=True, exist_ok=True)
        dst.write_text(rendered, encoding="utf-8")

    def render_html(self, src_path: Union[str, Path], context: Mapping=None) -> None:
        src_path = Path(src_path).expanduser()
        out_path = src_path.relative_to(self.src_dir)

        context = safe_context(context)
        original = Path(src_path).expanduser().read_text(encoding="utf-8")
        soup = bs4.BeautifulSoup(original, "html.parser")
        try:
            extract_head(soup, context)
            extract_main(soup, context)
            extract_scripts(soup, context)
        except ValueError as error:
            raise ValueError(f"While parsing '{str(src_path)}', encountered an error:") from error

        self.render_template(name="__partial.html", out_path=Path("_") / out_path, context=context)
        self.render_template(name="__full.html", out_path=out_path, context=context)

    def render_asset(self, src_path: Union[str, Path], out_path: Union[str, Path]=None):
        src_path = Path(src_path).expanduser()
        if out_path:
            out_path = Path(out_path).expanduser()
        else:
            out_path = src_path.relative_to(self.src_dir)
        dst = self.out_dir / out_path
        dst.parent.mkdir(parents=True, exist_ok=True)

        shutil.copy(str(src_path), str(dst))

    def process(self, context: Mapping=None) -> None:
        self.render_scaffolding(context=context)
        for src_path in self.src_dir.glob("**/*"):
            if src_path.is_dir():
                continue
            if src_path.suffix == ".html":
                self.render_html(src_path=src_path, context=context)
            else:
                self.render_asset(src_path=src_path)



import os
from pathlib import Path

from jinja2 import Environment, FileSystemLoader

from . import render

__version__ = "0.5.1"
GRITS_TEMPLATES = Path(os.path.abspath(os.path.dirname(__file__))).resolve() / "templates"
DEFAULT_JINJA_OPTIONS = {"trim_blocks": True, "lstrip_blocks": True}


def new_environment(template_dir: str, jinja_options: dict):
    searchpath = [str(GRITS_TEMPLATES)]
    if template_dir:
        # insert so users can override __full.html, __partial.html
        searchpath.insert(0, str(template_dir))
    loader = FileSystemLoader(searchpath)
    return Environment(loader=loader, **jinja_options)


def build(src_dir: str, out_dir: str, template_dir: str=None, context: dict=None, jinja_options: dict=None):
    environment = new_environment(
        template_dir,
        jinja_options=(jinja_options or dict(DEFAULT_JINJA_OPTIONS)))
    renderer = render.Renderer(environment=environment, src_dir=src_dir, out_dir=out_dir)
    renderer.process(context=context)
    return renderer



import click
import grits
existing_directory = click.Path(exists=True, dir_okay=True, file_okay=False)
directory = click.Path(exists=False, dir_okay=True, file_okay=False)


@click.command("build")
@click.option("--src", required=True, type=existing_directory, help="source directory")
@click.option("--dst", required=True, type=directory, help="output directory")
@click.option("--tpl", required=False, type=directory, help="templates directory")
def main(src, dst, tpl):
    grits.build(src_dir=src, out_dir=dst, template_dir=tpl)



import os
from setuptools import setup, find_packages


HERE = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(HERE, "README.rst")) as f:
    README = f.read()

with open(os.path.join(HERE, "CHANGELOG.rst")) as f:
    CHANGES = f.read()


def get_version():
    with open(os.path.join(HERE, "grits/__init__.py")) as f:
        for line in f:
            if line.startswith("__version__"):
                return eval(line.split("=")[-1])

REQUIREMENTS = [
    "beautifulsoup4==4.6.0",
    "click==6.7",
    "jinja2==2.9.5",
]

if __name__ == "__main__":
    setup(
        name="grits",
        version=get_version(),
        description="Static SPA Generator",
        long_description=README + "\n\n" + CHANGES,
        entry_points={
            "console_scripts": [
                "grits-build = grits.scripts.build:main",
                "grits-serve = grits.scripts.serve:main"
            ]
        },
        classifiers=[
            "Development Status :: 2 - Pre-Alpha",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.6",
        ],
        author="Joe Cross",
        author_email="joe.mcross@gmail.com",
        url="https://github.com/numberoverzero/grits",
        license="MIT",
        keywords="static blog spa generator",
        platforms="any",
        include_package_data=True,
        packages=find_packages(exclude=("test*", )),
        install_requires=REQUIREMENTS,
    )
